# coding=utf8
import sys
import os
import requests
import json
import base64
import matplotlib.pyplot as plt
from pylab import mpl
import math
import numpy as np
import operator
from mpl_toolkits.axisartist.parasite_axes import HostAxes, ParasiteAxes
import time

groupSession = ''
s = requests.session()
host = 'pump.iotn2n.com'
globalEnv = {}


"""完成拟合曲线参数计算前相应变量的计算"""
def polynomial_fitting(data_x,data_y):
    size=len(data_x)
    i=0
    sum_x = 0
    sum_sqare_x =0
    sum_third_power_x = 0
    sum_four_power_x = 0
    average_x = 0
    average_y = 0
    sum_y = 0
    sum_xy = 0
    sum_sqare_xy = 0
    while i<size:
        sum_x += data_x[i]
        sum_y += data_y[i]
        sum_sqare_x += math.pow(data_x[i],2)
        sum_third_power_x +=math.pow(data_x[i],3)
        sum_four_power_x +=math.pow(data_x[i],4)
        sum_xy +=data_x[i]*data_y[i]
        sum_sqare_xy +=math.pow(data_x[i],2)*data_y[i]
        i += 1;
    average_x=sum_x/size
    average_y=sum_y/size
    return [[size, sum_x, sum_sqare_x, sum_y]
        , [sum_x, sum_sqare_x, sum_third_power_x, sum_xy]
        , [sum_sqare_x,sum_third_power_x,sum_four_power_x,sum_sqare_xy]]


"""完成拟合曲线参数的计算
 其中解方程的时候，利用高斯消元法计算相应的参数值
"""
def calculate_parameter(data):
    #i用来控制列元素，line是一行元素,j用来控制循环次数,datas用来存储局部变量。保存修改后的值
    i = 0;
    j = 0;
    line_size = len(data)

    #将行列式变换为上三角行列式
    while j < line_size-1:
        line = data[j]
        temp = line[j]
        templete=[]
        for x in line:
            x=x/temp
            templete.append(x)
        data[j]=templete
        #flag标志应该进行消元的行数
        flag = j+1
        while flag < line_size:
            templete1 = []
            temp1=data[flag][j]
            i = 0
            for x1 in data[flag]:
                if x1!=0:
                    x1 = x1-(temp1*templete[i])
                    templete1.append(x1)
                else:
                    templete1.append(0)
                i += 1
            data[flag] = templete1
            flag +=1
        j += 1


    #求相应的参数值
    parameters=[]
    i=line_size-1
    #j标识减去元素个数
    #flag_rol标识除那一列
    flag_j=0
    rol_size=len(data[0])
    flag_rol=rol_size-2
    #获得解的个数
    while i>=0:
        operate_line = data[i]
        if i==line_size-1:
            parameter=operate_line[rol_size-1]/operate_line[flag_rol]
            parameters.append(parameter)
        else:
            flag_j=(rol_size-flag_rol-2)
            temp2=operate_line[rol_size-1]
            #result_flag为访问已求出解的标志
            result_flag=0
            while flag_j>0:
                temp2-=operate_line[flag_rol+flag_j]*parameters[result_flag]
                result_flag+=1
                flag_j-=1
            parameter=temp2/operate_line[flag_rol]
            parameters.append(parameter)
        flag_rol-=1
        i-=1
    return parameters

"""计算拟合曲线的值"""
def calculate(data_x,parameters):
    datay=[]
    for x in data_x:
        datay.append(parameters[2]+parameters[1]*x+parameters[0]*x*x)
    return datay

"""完成函数的绘制"""
def draw(flow, head, headparameters, power, powerparameters, effect, effectparameters):
    fm = math.ceil(max(flow))
    hm = math.ceil(max(head))
    hmin = math.floor(min(head))
    pm = math.ceil(max(power))
    pmin = math.floor(min(power))
    nm = math.ceil(max(effect))
    nmin = math.floor(min(effect))

    fig = plt.figure(1)
    host = HostAxes(fig, [0.15, 0.1, 0.65, 0.8])
    par1 = ParasiteAxes(host, sharex=host)
    par2 = ParasiteAxes(host, sharex=host)
    host.parasites.append(par1)
    host.parasites.append(par2)
    host.set_ylabel('扬程')
    host.set_xlabel('流量')
    host.axis['right'].set_visible(False)
    par1.axis['right'].set_visible(True)
    par1.set_ylabel('功率')

    par1.axis['right'].major_ticklabels.set_visible(True)
    par1.axis['right'].label.set_visible(True)

    par2.set_ylabel('扬程')
    offset = (60, 0)
    new_axisline = par2._grid_helper.new_fixed_axis
    par2.axis['right2'] = new_axisline(loc='right', axes=par2, offset=offset)
    fig.add_axes(host)
    host.set_xlim(0, fm+1)
    host.set_ylim(0, hm+5)

    host.set_xlabel('流量')
    host.set_ylabel('功率')
    host.set_ylabel('效率')
    x= np.linspace(0, fm, 500)
    y= headparameters[0] * x ** 2 + headparameters[1] * x + headparameters[2]
    p1,=host.plot(x, y,label="HQ拟合曲线",color="black")
    host.scatter(flow, head, label="HQ离散数据")
    x1 = np.linspace(0, fm, 500)
    y1 = powerparameters[0] * x ** 2 + powerparameters[1] * x + powerparameters[2]
    p2,=par1.plot(x, y1, label="PQ拟合曲线",color="red")
    par1.scatter(flow, power, label="PQ离散数据")
    x2= np.linspace(0, fm, 500)
    y2= effectparameters[0] * x ** 2 + effectparameters[1] * x + effectparameters[2]
    p3,=par2.plot(x, y2,label="EQ拟合曲线",color="blue")
    par2.scatter(flow, effect, label="EQ离散数据")
    par1.set_ylim(0, pm*2)
    par2.set_ylim(0, nm+5)
    host.legend()
    par2.axis['right2'].major_ticklabels.set_color(p3.get_color())  # 刻度值颜色
    par2.axis['right2'].set_axisline_style('-|>', size=1.5)  # 轴的形状色

    #解决使用matplotliblib画图的时候出现中文或者是负号无法显示的情况
    mpl.rcParams['font.sans-serif'] = ['SimHei']
    mpl.rcParams['axes.unicode_minus'] = False

    plt.title("性能曲线拟合数据")
    plt.legend(loc= "best")
    #获取当前时间
    localtime = time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime())
    print(localtime)
    filename = localtime + '.png'
    print(filename)
    plt.savefig(filename)
    # plt.show()
    return(filename)


def login():
    url = 'http://{}/iotapi/login'.format(host)
    headers = {'accept': 'application/json', "Content-Type": "text/plain"}
    body = {"username": 'jiance', "password": 'jiance'}
    r = s.post(url, headers=headers, data=json.dumps(body))
    global  globalEnv
    globalEnv = json.loads(r.content)  # 解码JSON对象
    global groupSession
    groupSession = r.json()['sessionToken']
    s.headers.update({"sessionToken": groupSession, 'Content-Type': 'application/json'})

def upload(file,session,state):
    role = state['roles'][0]['name']
    fileurl = state['roles'][0]['tag']['appconfig']['file']
    files = {'file': open(file, 'rb')}
    options = {'output': 'json', 'path': '', 'scene': role, 'auth_token':session}  # 参阅浏览器上传的选项
    uploadres = requests.post(fileurl,data=options, files=files)
    content = json.loads(uploadres.text)
    return content

def identify(flow, head, power, effect):
    #HG = json['hg']
    arr = []
    HG = 43
    QG = 10
    PG = 2.2
    nG = 72
    HGRC = 0.08
    QGRC = 0.05
    nGRC=-0.05
    headdata = polynomial_fitting(flow, head)
    headparameters = calculate_parameter(headdata)
    hq = headparameters[1] ** 2 - 4 * headparameters[0] * (headparameters[2] - HG)
    Qj = (-headparameters[1] - math.sqrt(hq)) / (2 * headparameters[0])
    Hj = headparameters[0] * QG ** 2 + headparameters[1] * QG + headparameters[2]
    HR = HG * HGRC
    QR = QG * QGRC
    if abs(Hj - HG) <= HR or abs(Qj - QG) <= QR:
        print("十字容差判定合格")
        ishegeh = 1
        item = {
            "tag": "十字容差判定合格",
            "source": "auto",
            "type": "text",
            "name": "噪声是否合格",
            "value": "合格"
        }
        arr.append(item)
    else:
        print("十字容差判定不合格")
        ishegeh = 0
        item = {
            "tag": "十字容差判定合格",
            "source": "auto",
            "type": "text",
            "name": "噪声是否合格",
            "value": "不合格"
        }
        arr.append(item)
    print("流量杨程合格判定", ishegeh)

    powerdata = polynomial_fitting(flow, power)
    powerparameters = calculate_parameter(powerdata)
    Wij = powerparameters[0] * QG ** 2 + powerparameters[1] * QG + powerparameters[2]
    pq = []
    for m in np.linspace(QG * 0.8, QG * 1.2, 1000):
        pq.append(powerparameters[0] * m ** 2 + powerparameters[1] * m + powerparameters[2])
    max_index, max_number = max(enumerate(pq), key=operator.itemgetter(1))
    if max_number <= PG:
        ishegep = 1
        item = {
            "tag": "功率判定合格",
            "source": "auto",
            "type": "text",
            "name": "功率是否合格",
            "value": "合格"
        }
        arr.append(item)
    else:
        ishegep = 0
        item = {
            "tag": "功率判定不合格",
            "source": "auto",
            "type": "text",
            "name": "功率是否合格",
            "value": "不合格"
        }
        arr.append(item)
    print("功率合格判定", ishegep)

    effectdata = polynomial_fitting(flow, effect)
    effectparameters = calculate_parameter(effectdata)
    nbj = effectparameters[0] * QG ** 2 + effectparameters[1] * QG + effectparameters[2]
    nR = nG * nGRC
    if nbj >= (nG + nR):
        ishegen = 1
    else:
        ishegen = 0
    print("效率合格判定", ishegen)
    print("流量实测值", Qj)
    print("扬程实测值", Hj)
    print("功率实测值", Wij)
    print("效率实测值", nbj)
    return arr

def post(args,session,env):
    body = json.loads(base64.b64decode(args).decode("utf-8"))
    print(body)
    state = json.loads(base64.b64decode(env).decode("utf-8"))
    print(state)
    resturl = state['roles'][0]['tag']['appconfig']['rest']
    print(resturl)
    s.headers.update({"sessionToken": session, 'Content-Type': 'application/json'})
    rt = s.get('{}/classes/Device?order=createdAt&limit=1&skip=0'.format(resturl),
                    params={'keys':'basedata','where': json.dumps({"objectId": body['name']})})
    for row in (rt.json()['results']):
       # print(row)
       print(row['basedata']['BasicInfo'])
       for item in row['basedata']['BasicInfo']:
           print(item)
           # text = {
           #     "tag": "性能曲线",
           #     "type": "image",
           #     "source": "auto",
           #     "name": item,
           #     "url": uploadfile['url'],
           #     "width": 600,
           #     "height": 330
           # }

    #url = 'http://{}:6020/rest/sql'.format(apihost)
    # curl -H 'Authorization: Basic <TOKEN>' -d '<SQL>' <ip>:<PORT>/rest/sql
    # TOKEN为{username}:{password}经过Base64编码之后的字符串，例如root:taosdata编码后为cm9vdDp0YW9zZGF0YQ==
    #headers = {'Authorization': 'Basic cm9vdDp0YW9zZGF0YQ=='}
    #data = "select flow, head, effect, power from _" + body['productid'] + "._" + body['deviceid'] + " order by createdat desc limit 20;"
    #r = requests.post(url, headers=headers,data=data)
    #t = r.content # 读取返回的内容
    #t = json.loads(t) # 解码JSON对象
    #flow = []
    #head = []
    #effect = []
    #power = []
    #for row in (t['data']):
    #    flow.append(row[0])
    #    head.append(row[1])
    #    effect.append(row[2])
    #    power.append(row[3])
    #print(flow)
    #print(head)
    #print(effect)
    #print(power)

    flow = [0.05,5.03,5.96,8.08,10.11,12.04,13.97]
    head = [55.89,53.07,52.2,48.85,43.92,37.24,29.6]
    power=[0.62,1.3,1.41,1.62,1.76,1.81,1.83]
    effect=[1.19,55.54,59.9,66.16,68.46,67.22,61.38]

    headdata = polynomial_fitting(flow,head)
    headparameters = calculate_parameter(headdata)

    powerdata = polynomial_fitting(flow,power)
    powerparameters = calculate_parameter(powerdata)

    effectdata = polynomial_fitting(flow,effect)
    effectparameters = calculate_parameter(effectdata)

    #print(body['reportid'])
    #print(state['username'])

    arr = []
    filename = draw(flow, head, headparameters, power, powerparameters, effect, effectparameters)
    uploadfile = upload(filename,session,state)
    print(uploadfile)

    image = {
        "tag": "性能曲线",
        "type": "image",
        "source": "auto",
        "name": "性能曲线",
        "url": uploadfile['url'],
        "width": 600,
        "height": 330
    }
    arr.append(image)

    print(image)
    result = identify(flow, head, power, effect)
    arr = arr + result
    print(json.JSONEncoder().encode(arr))
    # upload(file, state['username'], session)
    #return args

def main():
    global groupSession
    global globalEnv
    login()
    body = {
        "name": "211fc18573",
        "type": "中文"
    }
    body = json.JSONEncoder().encode(body)
    argvs = base64.b64encode(body.encode('utf-8'))
    print(argvs)

    env = json.JSONEncoder().encode(globalEnv)
    env = base64.b64encode(env.encode('utf-8'))
    print(env)

    return post(argvs, groupSession, env)

def exit():
    os._exit(0)


if __name__ == "__main__":
    main()